# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xlwt
import cStringIO
import base64
from .. import format_common

from account.report import account_financial_report
from openerp.osv import osv, fields
from tools.translate import _

class BS_PL_xls(osv.osv_memory):
    _inherit = "accounting.report"
    _description = 'Account Balance Sheet & Profit Loss Report'

    _columns = {'xls_output': fields.boolean('Excel Output', help='Tick if you want to export report in excel file.')}

    def check_report(self, cr, uid, ids, context=None):
        return super(BS_PL_xls, self).check_report(cr, uid, ids, context=context)

    def _print_report(self, cr, uid, ids, data, context=None):
        if context is None: context = {}
        reportobj = account_financial_report.report_account_common(cr, uid, 'report.account.common', context=context)
        res = super(BS_PL_xls, self)._print_report(cr, uid, ids, data, context=context)

        data['form'] = res['datas']['form']
        data['form'].update(self.read(cr, uid, ids, ['xls_output'])[0])
        if not data['form'].get('xls_output', False):
            return res

        GOLDEN_LIST = ['Name', 'Balance']
        GOLDEN_KEY = {'name':0, 'balance':1}

        if data['form'].get('debit_credit'):
            GOLDEN_LIST.extend(['Debit', 'Credit'])
            GOLDEN_KEY = {'name':0, 'debit':1, 'credit':2, 'balance':3}

        if data['form'].get('enable_filter'):
            data['form']['comparison_context'] = self._build_comparison_context(cr, uid, [], data, context)
            for key,val in data['form']['comparison_context'].items():
                if isinstance(val, tuple):
                    data['form']['comparison_context'][key] = val[0]

            label_name = data['form']['label_filter']
            GOLDEN_LIST.extend([label_name])
            GOLDEN_KEY = {'name':0, 'balance':1,'balance_cmp':2}

        filter_by = reportobj._get_filter(data)
        filter_by_title = 'No Filter'
        if filter_by == 'Date':
            filter_by_title = '%s-%s' % (reportobj._get_start_date(data), reportobj._get_end_date(data))
        elif filter_by == 'Periods':
            filter_by_title = '%s-%s' % (reportobj.get_start_period(data), reportobj.get_end_period(data))

        workbook = xlwt.Workbook()

        M_header_tstyle = format_common.font_style(position='center', bold=1, border=1, fontos='black', font_height=700)
        header_tstyle = format_common.font_style(position='left', bold=1, border=1, fontos='black', font_height=180, color='grey')
        header_tstyle_c = format_common.font_style(position='center', bold=1, border=1, fontos='black', font_height=180, color='grey')
        header_tstyle_r = format_common.font_style(position='right', bold=1, border=1, fontos='black', font_height=180, color='grey')
        view_tstyle = format_common.font_style(position='left', bold=1, fontos='black', font_height=180)
        view_tstyle_r = format_common.font_style(position='right', bold=1, fontos='black', font_height=180)
        other_tstyle = format_common.font_style(position='left', fontos='black', font_height=180)
        other_tstyle_c = format_common.font_style(position='center', fontos='black', font_height=180)
        other_tstyle_r = format_common.font_style(position='right', fontos='black', font_height=180)

        Hedaer_Text = 'Balance Sheet'
        if context and context.get('report'): Hedaer_Text = 'Profit And Loss'

        sheet = workbook.add_sheet(Hedaer_Text)
        sheet.row(0).height = 256 * 3
        sheet.write_merge(0, 0, 0, 3, Hedaer_Text, M_header_tstyle)

        sheet.write(3, 0, 'Chart of Account', header_tstyle_c)
        sheet.write(4, 0, reportobj._get_account(data) or '', other_tstyle_c)
        sheet.write(3, 1, 'Fiscal Year', header_tstyle_c)
        sheet.write(4, 1, reportobj._get_fiscalyear(data) or '' , other_tstyle_c)
        sheet.write(3, 2, 'Filter By', header_tstyle_c)
        sheet.write(4, 2, filter_by_title, other_tstyle_c)
        sheet.write(3, 3, 'Target Moves', header_tstyle_c)
        sheet.write(4, 3, reportobj._get_target_move(data), other_tstyle_c)
        sheet.col(0).width = 256 * 30
        sheet.col(1).width = 256 * 30
        sheet.col(2).width = 256 * 30
        sheet.col(3).width = 256 * 30

        for hd, ky in GOLDEN_KEY.items():
            Style = header_tstyle_r
            if hd.upper() == 'NAME':
                sheet.col(ky).width = 256 * 80
                Style = header_tstyle
            sheet.write(8, ky, hd.upper(), Style)

        def _apply_style(type, map):
            """
            Process:
                - Apply Stylesheet:
                    -Name : Left side(view)
                    -other coloums : right side
            """
            if type == 'view' or (not type):
                if map == 'name': Apply_style = view_tstyle
                else: Apply_style = view_tstyle_r
            else:
                if map == 'name': Apply_style = other_tstyle
                else: Apply_style = other_tstyle_r
            return Apply_style

        row = 9
        for val in reportobj.get_lines(data):
            if val['level'] == 0: continue
            for map, mapk in GOLDEN_KEY.items():
                if val.get(map):
                    if isinstance(val[map], unicode):
                        val[map] = ('   ' * val['level']) + val[map]
                    sheet.write(row, mapk, val[map], _apply_style(val['account_type'], map))
            row += 1

        stream = cStringIO.StringIO()
        workbook.save(stream)
        cr.execute(""" DELETE FROM output""")
        attach_id = self.pool.get('output').create(cr, uid, {'name':Hedaer_Text+'.xls', 'xls_output': base64.encodestring(stream.getvalue())})

        return {
                'name': _('Notification'),
                'context': context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'output',
                'res_id':attach_id,
                'type': 'ir.actions.act_window',
                'datas': data,
                'target':'new'
                }

BS_PL_xls()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
