# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import xlwt
import cStringIO
import base64
from .. import format_common

from openerp.osv import fields, osv
from account.report import account_balance
from openerp.tools.translate import _

class trial_balance_xls(osv.osv_memory):
    _inherit = "account.balance.report"

    _columns = {'xls_output': fields.boolean('Excel Output', help='Tick if you want to export report in excel file.')}

    def _print_report(self, cr, uid, ids, data, context=None):
        """
        Process
            - If "Excel output" mark as TRUE then generate EXCEL report or PDF report.
        """
        if context is None: 
            context = {}
        tbal_obj = account_balance.account_balance(cr, uid, 'account.account.balance', context=context)
        data = self.pre_print_report(cr, uid, ids, data, context=context)
        data['form'].update(self.read(cr, uid, ids, ['xls_output'])[0])

        # PDF REPORT
        if not data['form'].get('xls_output'): return {'type': 'ir.actions.report.xml', 'report_name': 'account.account.balance', 'datas': data}

        display_account = data['form']['display_account'] == 'all' and 'All' or  \
                            data['form']['display_account'] == 'movement' and 'With movements' \
                            or 'With balance is not equal to 0'

        filter_by = tbal_obj._get_filter(data)
        filter_by_title = 'No Filter'
        if filter_by == 'Date':
            filter_by_title = '%s-%s' % (tbal_obj._get_start_date(data), tbal_obj._get_end_date(data))
        elif filter_by == 'Periods':
            filter_by_title = '%s-%s' % (tbal_obj.get_start_period(data), tbal_obj.get_end_period(data))

        workbook = xlwt.Workbook()
        M_header_tstyle = format_common.font_style(position='center', bold=1, border=1, fontos='black', font_height=700)
        header_tstyle = format_common.font_style(position='left', bold=1, border=1, fontos='black', font_height=180, color='grey')
        header_tstyle_c = format_common.font_style(position='center', bold=1, border=1, fontos='black', font_height=180, color='grey')
        header_tstyle_r = format_common.font_style(position='right', bold=1, border=1, fontos='black', font_height=180, color='grey')
        view_tstyle = format_common.font_style(position='left', bold=1, fontos='black', font_height=180)
        view_tstyle_r = format_common.font_style(position='right', bold=1, fontos='black', font_height=180)
        other_tstyle = format_common.font_style(position='left', fontos='black', font_height=180)
        other_tstyle_c = format_common.font_style(position='center', fontos='black', font_height=180)
        other_tstyle_r = format_common.font_style(position='right', fontos='black', font_height=180)

        sheet = workbook.add_sheet('Trial Balance')
        sheet.row(0).height = 256 * 3
        sheet.write_merge(0, 0, 0, 4, 'Trial Balance', M_header_tstyle)

        sheet.write(3, 0, 'Chart of Account', header_tstyle_c)
        sheet.write(4, 0, tbal_obj._get_account(data) or '', other_tstyle_c)
        sheet.write(3, 1, 'Fiscal Year', header_tstyle_c)
        sheet.write(4, 1, tbal_obj._get_fiscalyear(data) or '' , other_tstyle_c)
        sheet.write(3, 2, 'Display Account', header_tstyle_c)
        sheet.write(4, 2, display_account , other_tstyle_c)
        sheet.write(3, 3, 'Filter By', header_tstyle_c)
        sheet.write(4, 3, filter_by_title, other_tstyle_c)
        sheet.write(3, 4, 'Target Moves', header_tstyle_c)
        sheet.write(4, 4, tbal_obj._get_target_move(data), other_tstyle_c)

        sheet.col(0).width = 256 * 30
        sheet.col(1).width = 256 * 50
        sheet.col(2).width = 256 * 30
        sheet.col(3).width = 256 * 30
        sheet.col(4).width = 256 * 30

        sheet.write(6, 0, 'Code', header_tstyle_c)
        sheet.write(6, 1, 'Account', header_tstyle_c)
        sheet.write(6, 2, 'Debit', header_tstyle_c)
        sheet.write(6, 3, 'Credit', header_tstyle_c)
        sheet.write(6, 4, 'Balance', header_tstyle_c)

        row = 7
        cell_count = 0
        for val in tbal_obj.lines(data['form'], ids=[data['form']['chart_account_id']]):
            for key in ['code', 'name', 'debit', 'credit', 'balance']:
                if val['type'] == 'view':
                    Style = view_tstyle
                    if key not in ('code','name'):
                        Style  = view_tstyle_r
                    sheet.write(row, cell_count, val[key], Style)
                else:
                    Style = other_tstyle
                    if key not in ('code','name'):
                        Style  = other_tstyle_r
                    sheet.write(row, cell_count, val[key], Style)
                cell_count += 1
            cell_count = 0
            row += 1

        stream = cStringIO.StringIO()
        workbook.save(stream)

        cr.execute(""" DELETE FROM output""")
        attach_id = self.pool.get('output').create(cr, uid, {'name':'Trial Balance.xls', 'xls_output':base64.encodestring(stream.getvalue())})
        return {
                'name': _('Notification'),
                'context': context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'output',
                'res_id':attach_id,
                'type': 'ir.actions.act_window',
                'target':'new'
                }

trial_balance_xls()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
